/***************************************************************************
 *   Copyright (C) 2003 by Stefano Rivoir                                  *
 *   s.rivoir@gts.it                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "kalsa.h"
#include "alsa/global.h"
#include "config.h"

#include <klocale.h>
#include "kamixconfig.h"

KAlsa *KAlsa::me = 0;

KAlsa::KAlsa()
{
}

KAlsa *KAlsa::self() {
    if ( !me )
        me = new KAlsa();
    return me;
}

KAlsa::~KAlsa()
{
    unsigned int i;
    for ( i=0; i<cards.count(); i++ )
        delete cards.at(i);
}

const char* KAlsa::version() {
#ifdef HAS_SND_ASOUNDLIB_VERSION
    return snd_asoundlib_version();
#else
    return i18n("ALSA version is not available");
#endif
}

void KAlsa::init() {
    unsigned int i;
    
    KCard *c;
    char *buf;
    QString name;
    QString hwName;

    for ( i=0; i<cards.count(); i++ )
        delete cards.at(i);
    cards.clear();
    
    for ( i=0; i<10; i++ ) {
        if ( !snd_card_load(i) ) {
	    hwName.sprintf ( "hw:%d",i ); 
	    snd_card_get_name ( i, &buf );
	    name = buf;
	    c = new KCard( hwName );
	    c->setName( name );
	    if ( c->openMixer() && c->mixer()->items.count()>0 )
	        cards.append(c);
	    else
	        delete ( c );
	} else
	    i = 99;
    }
}

unsigned int KAlsa::cardsCount() {
    return cards.count();
}

int KAlsa::shortItemId( int card ) {
    return KamixConfig::self()->rawLong( cards.at(card)->configSection(), "ShortItem", 0 );
}

KMixerItem* KAlsa::shortItem( int card ) {
    return cards.at(card)->mixer()->itemById( KamixConfig::self()->rawLong( cards.at(card)->configSection(), "ShortItem", 0 ) );
}

int KAlsa::defaultCard() {
    return KamixConfig::self()->defaultCard;
}

KCard* KAlsa::card(int i) {
    return cards.at(i);
}

#include "kalsa.moc"
