/***************************************************************************
 *   Copyright (C) 2003 by Stefano Rivoir                                  *
 *   s.rivoir@gts.it                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "kconfigwin.h"
#include "kmixeritem.h"
#include "kpreset.h"
#include "kamixconfig.h"
#include "kalsa.h"

#include <qlistbox.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <qframe.h>
#include <kapplication.h>
#include <klocale.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qptrlist.h>
#include <qtabwidget.h>
#include <qvgroupbox.h>
#include <qmap.h>
#include <kfontdialog.h>
#include <qvbox.h>
#include <kinputdialog.h>
#include <kmessagebox.h>

KConfigWin::KConfigWin(QWidget*)
 : KDialogBase( Tabbed, i18n("Options"), Ok|Cancel, Ok, 0, 0, true, true )
{

    activeCard = 0;

    QFrame *page1 = addPage( i18n("General") );
    QFrame *page5 = addPage( i18n("Appearance") );
    QFrame *page2 = addPage( i18n("Cards") );
    QFrame *page3 = addPage( i18n("Presets") );
    QFrame *page4 = addPage( i18n("OSD") );
    QFrame *page6 = addPage( i18n("Grouping") );
        
    QGridLayout *gl = new QGridLayout ( page1 );
    QGridLayout *gl2= new QGridLayout ( page2 );
    QGridLayout *gl3= new QGridLayout ( page3 );
    QGridLayout *gl4= new QGridLayout ( page4 );
    QGridLayout *gl5= new QGridLayout ( page5 );
    QVBoxLayout *vbg2 = new QVBoxLayout ( page6 );
    QGridLayout *gl62= new QGridLayout ( vbg2 );
    QGridLayout *gl6= new QGridLayout ( vbg2 );
    
    QVGroupBox *gbox = new QVGroupBox ( i18n("General options"), page1 );
        
    /*
        General options
    */
    cLoad = new QCheckBox(i18n("Load levels on startup"), gbox );
    cSave = new QCheckBox(i18n("Save levels on exit"), gbox );
    cHideI= new QCheckBox(i18n("Hide inactive controls"), gbox );
    cPG   = new QCheckBox(i18n("Restore window geometry and position"), gbox );
    cAST  = new QCheckBox(i18n("Always restore to systray"), gbox );
    cDebug= new QCheckBox(i18n("Debug info when launched from console"), gbox );
    cMid  = new QCheckBox(i18n("Mid button on systray toggles mute"), gbox );
    
    vu    = new KKeyButton( page1 );
    vd    = new KKeyButton( page1 );
    QLabel *lu = new QLabel(i18n("Master vol &up key"), page1 );
    QLabel *ld = new QLabel(i18n("Master vol &down key"), page1 );
    lu->setBuddy ( vu );
    ld->setBuddy ( vd );
    
    connect ( vu, SIGNAL(capturedShortcut(const KShortcut&)), 
    	this, SLOT(slotVolUpKey(const KShortcut&)) );
    connect ( vd, SIGNAL(capturedShortcut(const KShortcut&)), 
    	this, SLOT(slotVolDnKey(const KShortcut&)) );

    cSave->setChecked ( KamixConfig::self()->saveLevels );
    cLoad->setChecked ( KamixConfig::self()->loadLevels );
    cPG->setChecked   ( KamixConfig::self()->restoreWindow );
    cDebug->setChecked( KamixConfig::self()->debugInfo );
    cHideI->setChecked( KamixConfig::self()->hideInactive );
    cAST->setChecked  ( KamixConfig::self()->alwaysSystray );
    cMid->setChecked ( KamixConfig::self()->midButtonMutes );
    
    vu->setShortcut ( KShortcut ( KamixConfig::self()->volUpShortcut ), false );
    vd->setShortcut ( KShortcut ( KamixConfig::self()->volDnShortcut ), false );
    
    gl->addMultiCellWidget ( gbox, 0, 0, 0, 1 );
    
    gl->addWidget ( lu, 1, 0 );
    gl->addWidget ( vu, 1, 1 );
    gl->addWidget ( ld, 2, 0 );
    gl->addWidget ( vd, 2, 1 );
    
    /*
        Appearance
    */
    
    cTick = new QCheckBox(i18n("Show tickmarks on sliders"), page5 ) ;
    cMute = new QCheckBox(i18n("Show mute buttons"), page5 );
    cBal  = new QCheckBox(i18n("Show balance buttons"), page5 );
    cFrames=new QCheckBox(i18n("Show frame around items"), page5 );
    cGFrames=new QCheckBox(i18n("Show frame around groups"), page5 );
    
    QLabel *lblLFont = new QLabel ( i18n("Channels/cards font"), page5 );
    bLblFont = new QPushButton ( i18n("Choose"), page5 );
    
    cTick->setChecked ( KamixConfig::self()->showTicks );
    cMute->setChecked ( KamixConfig::self()->showMute );
    cBal->setChecked  ( KamixConfig::self()->showBal );
    cFrames->setChecked(KamixConfig::self()->itemsFrame );
    cGFrames->setChecked ( KamixConfig::self()->showFrameAroundGroups );
    
    lblf = &KamixConfig::self()->lblFont;
    gl5->addMultiCellWidget ( cTick, 0, 0, 0, 1 );
    gl5->addMultiCellWidget ( cMute, 1, 1, 0, 1 );
    gl5->addMultiCellWidget ( cBal , 2, 2, 0, 1 );
    gl5->addMultiCellWidget ( cFrames, 3, 3, 0, 1 );
    gl5->addMultiCellWidget ( cGFrames, 4, 4, 0, 1 );
    gl5->addWidget ( lblLFont, 5, 0 );
    gl5->addWidget ( bLblFont, 5, 1 );
    
    /*
        Per-card config
    */
    
    QLabel *sh = new QLabel ( i18n("Item to show in short view"), page2 );
    cShort= new QComboBox( page2 );
    sh->setBuddy ( cShort );
    
    mCard  = new QCheckBox ( i18n("Default card"), page2 );
    cCards = new QComboBox ( page2 );

    QString s;
    unsigned int i;
    for ( i=0; i<KAlsa::self()->cardsCount(); i++ ) {
        s.setNum( i );
	s+=". ";
	s+=KAlsa::self()->card(i)->name();
        cCards->insertItem( s );
    }
    
    lst = new QListView ( page2 );
    lst->addColumn ( i18n("Items") );
    lst->addColumn ( i18n("Item ID") );
    lst->setSorting ( 0, false );
    lst->setSorting ( 1 );

    gl2->addWidget ( cCards, 0, 0 );
    gl2->addWidget ( mCard, 1, 0 );
    gl2->addWidget ( sh, 2, 0 );
    gl2->addWidget ( cShort, 3, 0 );
    gl2->addWidget ( lst, 4, 0 );

    connect ( cCards, SIGNAL(activated(int)), this, SLOT(changeActiveCard()) );
    fillItems();
    
    /*
        Presets
    */
    
    QVBoxLayout *ly2 = new QVBoxLayout ( gl3 );
    pst = new QListView ( page3 );
    pst->addColumn ( i18n("Presets" ) );
    bSavePst = new QPushButton ( i18n("Save"), page3 );
    bDelPst = new QPushButton ( i18n("Delete"), page3 );
    bApplyPst = new QPushButton ( i18n("Apply"), page3 );
    
    QLabel *lname = new QLabel ( i18n("Save current volumes as preset:"), page3 );
    
    bSavePst->setEnabled ( false );
    
    pname = new QLineEdit ( page3 );
    
    connect ( pname, SIGNAL(textChanged(const QString&)), this, SLOT(pnameChanged(const QString&)) );
    connect ( bSavePst, SIGNAL(pressed()), this, SLOT(savePreset()) );
    connect ( bApplyPst, SIGNAL(pressed()), this, SLOT(applyPreset()) );
    connect ( bDelPst, SIGNAL(pressed()), this, SLOT(delPreset()) );
    connect ( pst, SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(activePresetChanged(QListViewItem*) ) );
    connect ( pst, SIGNAL(doubleClicked ( QListViewItem *, const QPoint &, int )), this, SLOT(applyPreset()) );
    
    gl3->addWidget ( pst, 0, 0 );
    gl3->addWidget ( lname, 1, 0 );
    gl3->addWidget ( pname, 2, 0 );
    gl3->addWidget ( bSavePst, 3, 0 );
    
    ly2->addWidget ( bApplyPst );
    ly2->addWidget ( bDelPst );
    
    gl3->addMultiCell( ly2, 0, 0, 1, 1, Qt::AlignTop );
    
    fillPresets();
    
    /*
        OSD
    */
    
    QLabel *osd3 = new QLabel ( i18n("OSD position"), page4 );
    
    cOsd = new QCheckBox ( i18n("Show OSD messages"), page4 );
    ceOsd= new QCheckBox ( i18n("Show only when externally triggered"), page4 );
    cop = new QComboBox ( page4 );
    QLabel *fname = new QLabel ( i18n("OSD font"), page4 );
    bFont = new QPushButton ( i18n("Choose"), page4 );
    cbb = new KColorButton ( page4 );
    cbf = new KColorButton ( page4 );
    QLabel *l1 = new QLabel ( i18n("Background color"), page4 );
    QLabel *l2 = new QLabel ( i18n("Foreground color"), page4 );
    QLabel *l3 = new QLabel ( i18n("X position"), page4 );
    QLabel *l4 = new QLabel ( i18n("Y position"), page4 );
    osdX = new QSpinBox ( 10, QApplication::desktop()->screen(0)->size().width()-10, 1, page4 );
    osdY = new QSpinBox ( 10, QApplication::desktop()->screen(0)->size().height()-10, 1, page4 );
    
    osdf = &KamixConfig::self()->osdFont;
    
    cop->insertItem ( i18n("Top left") );
    cop->insertItem ( i18n("Top right") );
    cop->insertItem ( i18n("Bottom left") );
    cop->insertItem ( i18n("Bottom right") );
    cop->insertItem ( i18n("Custom") );
    
    gl4->addMultiCellWidget ( cOsd , 0, 0, 0, 1 );
    gl4->addMultiCellWidget ( ceOsd, 1, 1, 0, 1 );
    gl4->addWidget ( osd3,  2, 0 );
    gl4->addWidget ( cop ,  2, 1 );
    gl4->addWidget ( l3, 3, 0 );
    gl4->addWidget ( osdX, 3, 1 );
    gl4->addWidget ( l4, 4, 0 );
    gl4->addWidget ( osdY, 4, 1 );
    gl4->addWidget ( fname, 5, 0 );
    gl4->addWidget ( bFont, 5, 1 );
    gl4->addWidget ( l1, 6, 0 );
    gl4->addWidget ( cbb, 6, 1 );
    gl4->addWidget ( l2, 7, 0 );
    gl4->addWidget ( cbf, 7, 1 );
    
    cbb->setColor ( KamixConfig::self()->osdBackcolor );
    cbf->setColor ( KamixConfig::self()->osdForecolor );
    
    cop->setCurrentItem( KamixConfig::self()->osdPosition );
    cOsd->setChecked ( KamixConfig::self()->showOSD );
    ceOsd->setChecked ( KamixConfig::self()->showOSDExtern );
    osdX->setValue ( KamixConfig::self()->osdPositionX );
    osdY->setValue ( KamixConfig::self()->osdPositionY );
    
    connect ( bFont, SIGNAL(pressed()), this, SLOT(slotChooseOSDFont()) );
    connect ( bLblFont, SIGNAL(pressed()), this, SLOT(slotChooseLBLFont()) );
    
    /*
        Grouping
    */
    
    QLabel *lg1 = new QLabel ( i18n("Card"), page6 );
    QLabel *lg2 = new QLabel ( i18n("Group"), page6 );
    QLabel *lg3 = new QLabel ( i18n("Mode"), page6 );
    
    cCards2= new QComboBox ( page6 );
    cGroup = new QComboBox ( page6 );
    cGroupMode = new QComboBox ( page6 );
    
    newgroup = new QPushButton ( i18n("New group"), page6 );
    delgroup = new QPushButton ( i18n("Delete group"), page6 );
    
    gl62->addWidget ( lg1, 0, 0 );
    gl62->addWidget ( lg3, 1, 0 );
    gl62->addWidget ( lg2, 2, 0 );
    gl62->addMultiCellWidget ( cCards2, 0, 0, 1, 2 );
    gl62->addMultiCellWidget ( cGroupMode, 1, 1, 1, 2 );
    gl62->addMultiCellWidget ( cGroup, 2, 2, 1, 2 );
    gl62->addWidget ( newgroup, 3, 1 );
    gl62->addWidget ( delgroup, 3, 2 );
    
    cGroupMode->insertItem( i18n("Playback") );
    cGroupMode->insertItem( i18n("Capture") );
    
    QVBox *vbg1 = new QVBox(page6);
    addg = new QPushButton( "<", vbg1 );
    
    QLabel *lg4 = new QLabel ( i18n("In group"), page6 );
    QLabel *lg5 = new QLabel ( i18n("Not in group"), page6 );
    ing = new QListBox(page6);
    ning= new QListBox(page6);
    
    for ( i=0; i<KAlsa::self()->cardsCount(); i++ ) {
        s.setNum( i );
	s+=". ";
	s+=KAlsa::self()->card(i)->name();
        cCards2->insertItem( s );
    }
    
    gl6->addWidget ( lg4, 3, 0 );
    gl6->addWidget ( lg5, 3, 2 );
    gl6->addWidget ( ing, 4, 0 );
    gl6->addWidget ( ning, 4, 2 );
    
    gl6->addWidget ( vbg1, 4, 1 );
    
    fillGroups();
    changeGroup();
    fillGroupsComp();
    
    connect ( cCards, SIGNAL(activated(int)), this, SLOT(changeActiveCard()) );
    connect ( cCards2, SIGNAL(activated(int)), this, SLOT(changeActiveCard2()) );
    connect ( cGroup, SIGNAL(activated(int)), this, SLOT(changeGroup()) );
    connect ( cGroupMode, SIGNAL(activated(int)), this, SLOT(changeGroupMode()) ); 
    connect ( addg, SIGNAL(clicked()), this, SLOT(addInGroup()) );
    connect ( newgroup, SIGNAL(clicked()), this, SLOT(newGroup()) );
    connect ( delgroup, SIGNAL(clicked()), this, SLOT(delGroup()) );

}

void KConfigWin::delGroup()
{
    if ( KMessageBox::questionYesNo(this, i18n("Are you sure you want to delete this group?")) == No )
        return;
    
    int dg = cGroup->currentItem();
    
    QStringList str;    
    if ( cGroupMode->currentItem() == 0 )
	str = KamixConfig::self()->playbackGroups ( configSect );
    else
	str = KamixConfig::self()->captureGroups ( configSect );

    str.remove(cGroup->currentText());
    if ( cGroupMode->currentItem() == 0 )
        KamixConfig::self()->setRawList ( configSect, "PlaybackGroups", str );
    else
        KamixConfig::self()->setRawList ( configSect, "CaptureGroups", str );

    unsigned int i;
    int g;
    QString s;
    for ( i=0 ; i<KAlsa::self()->card(activeCard)->mixer()->items.count() ; i++ ) {
        
	//s.sprintf ( (cGroupMode->currentItem()==0 ? "PlaybackGroup_%d" : "CaptureGroup_%d" ), i );
	//g = conf->readNumEntry ( s, 0 );
	
	if ( cGroupMode->currentItem()==0 )
	    s = "PlaybackGroup";
	else
	    s = "CaptureGroup";
	    
	g = KamixConfig::self()->rawLong ( configSect, s, 0, i );
	
	if ( g==dg )
	    KamixConfig::self()->setRawLong ( configSect, s, 0 );
	    //conf->writeEntry ( s, 0 );
	if ( g>dg )
	    KamixConfig::self()->setRawLong ( configSect, s, g-1 );
	    //conf->writeEntry ( s, g-1 );
    }
        
    fillGroups();
    fillGroupsComp();
    
}

void KConfigWin::newGroup() {
    QString s = KInputDialog::getText(i18n("New group"),i18n("Insert new group name") );
    if ( s.contains(",") )
        KMessageBox::sorry( this, i18n("Group names cannot contain commas") );
    if ( s == "" )
        return;
    
    QStringList str;    
    if ( cGroupMode->currentItem() == 0 )
	str = KamixConfig::self()->playbackGroups ( configSect );
    else
	str = KamixConfig::self()->captureGroups ( configSect );
    str.append ( s );

    if ( cGroupMode->currentItem() == 0 )
        KamixConfig::self()->setRawList ( configSect, "PlaybackGroups", str );
    else
        KamixConfig::self()->setRawList ( configSect, "CaptureGroups", str );
    
    fillGroups();
    fillGroupsComp();
    
}

void KConfigWin::addInGroup() {

    if ( ning->currentItem() == -1 )
        return;

    QString s = ning->currentText();
    KMixerItem *itm = KAlsa::self()->card(activeCard)->mixer()->itemByName( s );
    
    if ( cGroupMode->currentItem()==0 )
	s = "PlaybackGroup";
    else
	s = "CaptureGroup";

    KamixConfig::self()->setRawLong ( configSect, s, cGroup->currentItem(), itm->id );
    fillGroupsComp();
}

void KConfigWin::changeActiveCard() {
    saveCardSettings();
    activeCard = cCards->currentItem();
    configSect = KAlsa::self()->card(activeCard)->configSection();
    fillItems();
}

void KConfigWin::changeGroupMode() {
    fillGroups();
    fillGroupsComp();
}

void KConfigWin::changeActiveCard2() {
    activeCard = cCards2->currentItem();
    configSect = KAlsa::self()->card(activeCard)->configSection();
    fillGroups();
    fillGroupsComp();
}

void KConfigWin::fillItems() {
    unsigned int i;
    QString s; //, s2;
    QCheckListItem *it;
    
    KMixer *mix = KAlsa::self()->card(activeCard)->mixer();
    
    if ( KAlsa::self()->cardsCount() == 1 ) {
        mCard->setChecked ( true );
	mCard->setEnabled ( false );
    } else {
	mCard->setChecked ( KamixConfig::self()->defaultCard == activeCard );
	mCard->setEnabled ( true );
    }
    
    lst->clear();
    iP = new QListViewItem ( lst, i18n("Playback") );
    iC = new QListViewItem ( lst, i18n("Capture") );
    iC->setOpen ( true );
    iP->setOpen ( true );
    
    for ( i=0; i<mix->items.count(); i++ ) {
        it = 0;
        if ( mix->items.at(i)->canPlayback ) {
            it =  new QCheckListItem ( iP, mix->items.at(i)->name(), QCheckListItem::CheckBox );
	    it->setText ( 1, s.sprintf("%d", i) );
	    it->setOn ( KamixConfig::self()->showPlayback( configSect, i ) );
	    cShort->insertItem ( mix->items.at(i)->name() );
	}
        if ( mix->items.at(i)->canCapture ) {
            it =  new QCheckListItem ( iC, mix->items.at(i)->name(), QCheckListItem::CheckBox );
	    it->setText ( 1, s.sprintf("%d", i) );
	    it->setOn ( KamixConfig::self()->showCapture( configSect, i ) );
	}
    }
    
    if ( !KamixConfig::self()->rawLong( s, "ShortItem", 0 ) ) {
        cShort->setCurrentText ( mix->items.at(0)->name() );
    } else {
	cShort->setCurrentText ( mix->itemById( KamixConfig::self()->rawLong ( s, "ShortItem", 0 ) )->name() );
    }

}

void KConfigWin::fillPresets() {
    pst->clear();
    unsigned int i;
    for ( i=0; i<KPreset::count(); i++ ) {
        pitm = new QListViewItem ( pst, KPreset::preset(i) );
    }
    if ( pst->currentItem()==0 ) {
        bApplyPst->setEnabled(false);
	bDelPst->setEnabled(false);
    } else {
        bApplyPst->setEnabled(true);
	bDelPst->setEnabled(true);
    }
    
    pname->setText("");
}

void KConfigWin::fillGroupsComp() {
    ing->clear();
    ning->clear();
    
    unsigned int i;
    int g;
    QString s;
    
    KMixerItem *itm;
    for ( i=0; i<KAlsa::self()->card(activeCard)->mixer()->elemCount() ; i++ ) {
        itm = KAlsa::self()->card(activeCard)->mixer()->itemById(i);
	if ( ( itm && itm->canPlayback && cGroupMode->currentItem()==0 ) ||
	     ( itm && itm->canCapture  && cGroupMode->currentItem()==1 ) ) {
	     
	     if ( cGroupMode->currentItem()==0 )
		 s = "PlaybackGroup";
             else
		 s = "CaptureGroup";
	     
	     g = KamixConfig::self()->rawLong ( configSect, s, 0 );
	     if ( g==cGroup->currentItem() )
	         ing->insertItem ( itm->name() );
             else
	         ning->insertItem ( itm->name() );
	}
    }
}

void KConfigWin::changeGroup() {
    delgroup->setEnabled( cGroup->currentItem() != 0 );
    fillGroupsComp();
}

void KConfigWin::fillGroups() {
    cGroup->clear();
    
    QStringList str;    
    if ( cGroupMode->currentItem() == 0 )
	str = KamixConfig::self()->playbackGroups ( configSect );
    else
	str = KamixConfig::self()->captureGroups ( configSect );
    str.prepend ( i18n("Main") );
    cGroup->insertStringList ( str );
    delgroup->setEnabled( cGroup->currentItem() != 0 );
}

KConfigWin::~KConfigWin()
{
}

void KConfigWin::slotOk()
{
    saveCardSettings();
    
    KamixConfig::self()->volDnShortcut = vd->shortcut().toString();
    KamixConfig::self()->volUpShortcut = vu->shortcut().toString();
    KamixConfig::self()->showTicks = cTick->isChecked();
    KamixConfig::self()->showMute  = cMute->isChecked();
    KamixConfig::self()->showBal   = cBal->isChecked();
    KamixConfig::self()->loadLevels= cLoad->isChecked();
    KamixConfig::self()->saveLevels= cSave->isChecked();
    KamixConfig::self()->restoreWindow = cPG->isChecked();
    KamixConfig::self()->debugInfo = cDebug->isChecked();
    KamixConfig::self()->hideInactive = cHideI->isChecked();
    KamixConfig::self()->itemsFrame = cFrames->isChecked();
    KamixConfig::self()->showFrameAroundGroups = cGFrames->isChecked();
    KamixConfig::self()->alwaysSystray = cAST->isChecked();
    KamixConfig::self()->showOSD = cOsd->isChecked();
    KamixConfig::self()->showOSDExtern = ceOsd->isChecked();
    KamixConfig::self()->osdPosition = cop->currentItem();
    KamixConfig::self()->osdPositionX = osdX->value();
    KamixConfig::self()->osdPositionY = osdY->value();
    KamixConfig::self()->osdFont = *osdf;
    KamixConfig::self()->lblFont = *lblf;
    KamixConfig::self()->osdBackcolor = cbb->color();
    KamixConfig::self()->osdForecolor = cbf->color();
    KamixConfig::self()->midButtonMutes = cMid->isChecked();
    
    KamixConfig::self()->writeConfig();
    
    KDialogBase::slotOk();
}

void KConfigWin::saveCardSettings() {

    QCheckListItem *itm;
    itm = (QCheckListItem*)iP->firstChild();
    while ( itm ) {
	KamixConfig::self()->setShowPlayback ( configSect, itm->text(1).toLong(), itm->isOn() );
	itm = (QCheckListItem*)itm->nextSibling();
    }

    itm = (QCheckListItem*)iC->firstChild();
    while ( itm ) {
	KamixConfig::self()->setShowCapture ( configSect, itm->text(1).toLong(), itm->isOn() );
	itm = (QCheckListItem*)itm->nextSibling();
    }

    KamixConfig::self()->setRawLong ( configSect, "ShortItem", 
        KAlsa::self()->card(activeCard)->mixer()->itemByName( cShort->currentText() )->id );
    
    if ( mCard->isChecked() ) {
	KamixConfig::self()->defaultCard = activeCard;
    }
}

void KConfigWin::savePreset() {
    QString s = KPreset::volumesToString();
    KamixConfig::self()->setPreset ( pname->text(), s );
    fillPresets();
}

void KConfigWin::delPreset() {
    KamixConfig::self()->delPreset ( pst->currentItem()->text(0) );
    fillPresets();
}

void KConfigWin::applyPreset() {
    KPreset::loadPreset( pst->currentItem()->text(0) );
}

void KConfigWin::pnameChanged(const QString& s) {
    bSavePst->setEnabled ( !s.isEmpty() );
}

void KConfigWin::slotVolUpKey( const KShortcut &sc ) {
    vu->setShortcut ( sc, false );
}

void KConfigWin::slotVolDnKey( const KShortcut &sc ) {
    vd->setShortcut ( sc, false );
}

void KConfigWin::activePresetChanged(QListViewItem* itm ) {
    pname->setText( itm->text(0) );
}

void KConfigWin::slotChooseOSDFont() {
    KFontDialog::getFont( *osdf, false, this );
}

void KConfigWin::slotChooseLBLFont() {
    KFontDialog::getFont( *lblf, false, this );
}

#include "kconfigwin.moc"
