/***************************************************************************
 *   Copyright (C) 2003 by Stefano Rivoir                                  *
 *   s.rivoir@gts.it                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "kgroupwidget.h"
#include <qpainter.h>
#include <qpushbutton.h>
#include <qlayout.h>

#include "kamixconfig.h"

KGroupWidget::KGroupWidget(QWidget *parent, const char *name, bool e )
  : QWidget ( parent, name )
{

    QVBoxLayout *lo = new QVBoxLayout(this);
    
    vbox = new QVBox (this);
    
    showFrame = KamixConfig::self()->showFrameAroundGroups;
    
    if ( showFrame )
        vbox->setFrameShape ( QFrame::Panel );
    
    _expanded = e;
    
    lhead = new KVertLabel ( vbox, KVertLabel::GroupShown );
    
    hboxb = new QHBox ( vbox );
    hbox = new QHBox ( hboxb );
    hbox->setSpacing(5);
    
    lside = new KVertLabel(hboxb, KVertLabel::GroupHidden );
    
    connect ( lside, SIGNAL(clicked()), this, SLOT(slotToggle() ) );
    connect ( lhead, SIGNAL(clicked()), this, SLOT(slotToggle() ) );
    
    paintEvent(NULL);
    
    lo->addWidget ( vbox );
}

KGroupWidget::~KGroupWidget()
{
}

bool KGroupWidget::expanded() {
    return _expanded;
}

void KGroupWidget::setExpanded(bool b) {
    _expanded = b;
    update();
}

void KGroupWidget::setTitle( const QString &t ) 
{
    lside->setText ( t );
    lhead->setText ( t );
}

void KGroupWidget::paintEvent(QPaintEvent* ev) {
    if ( _expanded ) {
        hbox->show();
	lside->hide();
	lhead->show();
	if ( showFrame )
	    vbox->setFrameShape ( QFrame::Panel );
    } else {
        hbox->hide();
	lside->show();
	lhead->hide();
	vbox->setFrameShape ( QFrame::NoFrame );
    }
    QWidget::paintEvent(ev);
}

void KGroupWidget::slotToggle() {
    _expanded = !_expanded;    
    update();
}

#include "kgroupwidget.moc"
